package org.simantics.scl.compiler.constants;

import org.cojen.classfile.TypeDesc;
import org.simantics.scl.compiler.internal.codegen.references.Val;
import org.simantics.scl.compiler.internal.codegen.utils.Constants;
import org.simantics.scl.compiler.internal.codegen.utils.MethodBuilder;
import org.simantics.scl.compiler.types.TVar;
import org.simantics.scl.compiler.types.Type;
import org.simantics.scl.compiler.types.Types;

public class StringInterpolation extends FunctionValue {

    public static final TypeDesc STRING_BUILDER = TypeDesc.forClass(StringBuilder.class);
    
    private final String[] textParts;
    
    public StringInterpolation(Type[] parameterTypes, String[] textParts) {
        super(TVar.EMPTY_ARRAY, Types.NO_EFFECTS, Types.STRING, parameterTypes);
        this.textParts = textParts;
    }
    
    public StringInterpolation(String[] textParts) {
        this(stringTypeArray(textParts.length-1), textParts);
    }

    private static Type[] stringTypeArray(int length) {
        Type[] result = new Type[length];
        for(int i=0;i<length;++i)
            result[i] = Types.STRING;
        return result;
    }

    @Override
    public Type applyExact(MethodBuilder mb, Val[] parameters) {
        mb.newObject(STRING_BUILDER);
        mb.dup();
        mb.invokeConstructor(STRING_BUILDER, Constants.EMPTY_TYPEDESC_ARRAY);
        
        for(int i=0;i<parameters.length;++i) {
            String textPart = textParts[i]; 
            if(!textPart.isEmpty()) {
                mb.loadConstant(textPart);
                mb.invokeVirtual(STRING_BUILDER, "append", STRING_BUILDER, new TypeDesc[] {TypeDesc.STRING});
            }
            
            mb.push(parameters[i], parameterTypes[i]);
            mb.invokeVirtual(STRING_BUILDER, "append", STRING_BUILDER,
                    new TypeDesc[] {mb.getJavaTypeTranslator().toTypeDesc(parameterTypes[i])});
        }
        
        {
            String textPart = textParts[parameters.length]; 
            if(!textPart.isEmpty()) {
                mb.loadConstant(textPart);
                mb.invokeVirtual(STRING_BUILDER, "append", STRING_BUILDER, new TypeDesc[] {TypeDesc.STRING});
            }
        }
        
        mb.invokeVirtual(STRING_BUILDER, "toString", TypeDesc.STRING, Constants.EMPTY_TYPEDESC_ARRAY);
        
        return Types.STRING;
    }

}
